package ru.dbotthepony.mc.otm.capability;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraftforge.common.util.INBTSerializable;

import javax.annotation.Nonnull;
import java.math.BigDecimal;

public class MatteryMachineEnergyStorage implements IMatteryEnergyStorage, INBTSerializable<CompoundTag> {
	public enum MachineType {
		WORKER,
		GENERATOR,
		CAPACITOR,
	}

	public static final BigDecimal DEFAULT_MAX_RECEIVE = new BigDecimal(200);
	public static final BigDecimal DEFAULT_MAX_EXTRACT = new BigDecimal(200);
	public static final BigDecimal DEFAULT_MAX_CAPACITY = new BigDecimal(60000);

	protected BigDecimal energy_stored = BigDecimal.ZERO;
	protected BigDecimal energy_stored_max;
	protected BigDecimal max_input;
	protected BigDecimal max_output;
	protected final MachineType machine_type;
	protected final BlockEntity listener;

	public MatteryMachineEnergyStorage(BlockEntity listener, MachineType type, BigDecimal capacity) {
		this(listener, type, capacity,  DEFAULT_MAX_RECEIVE, DEFAULT_MAX_EXTRACT);
	}

	public MatteryMachineEnergyStorage(BlockEntity listener, MachineType type) {
		this(listener, type, DEFAULT_MAX_CAPACITY);
	}

	public MatteryMachineEnergyStorage(BlockEntity listener, MachineType type, BigDecimal capacity, BigDecimal maxReceive, BigDecimal maxExtract) {
		this.listener = listener;
		energy_stored_max = capacity;
		max_input = maxReceive;
		max_output = maxExtract;
		machine_type = type;
	}

	@Nonnull
	@Override
	public BigDecimal extractEnergyOuter(BigDecimal howMuch, boolean simulate) {
		if (machine_type == MachineType.WORKER) {
			return BigDecimal.ZERO;
		}

		return extractEnergyInner(howMuch, simulate);
	}

	@Nonnull
	@Override
	public BigDecimal extractEnergyInner(BigDecimal howMuch, boolean simulate) {
		BigDecimal new_energy = energy_stored.subtract(howMuch.min(max_output), MatteryCapability.ROUND_RULES).max(BigDecimal.ZERO);
		BigDecimal diff = energy_stored.subtract(new_energy);

		if (!simulate && !energy_stored.equals(new_energy)) {
			energy_stored = new_energy;
			listener.setChanged();
		}

		return diff;
	}

	@Nonnull
	@Override
	public BigDecimal receiveEnergyOuter(BigDecimal howMuch, boolean simulate) {
		if (machine_type == MachineType.GENERATOR) {
			return BigDecimal.ZERO;
		}

		return receiveEnergyInner(howMuch, simulate);
	}

	@Nonnull
	@Override
	public BigDecimal receiveEnergyInner(BigDecimal howMuch, boolean simulate) {
		BigDecimal new_energy = energy_stored.add(howMuch.min(max_input), MatteryCapability.ROUND_RULES).min(energy_stored_max);
		BigDecimal diff = new_energy.subtract(energy_stored);

		if (!simulate && !energy_stored.equals(new_energy)) {
			energy_stored = new_energy;
			listener.setChanged();
		}

		return diff;
	}

	@Nonnull
	@Override
	public BigDecimal getBatteryLevel() {
		return energy_stored;
	}

	@Nonnull
	@Override
	public BigDecimal getMaxBatteryLevel() {
		return energy_stored_max;
	}

	@Override
	public boolean canExtract() {
		return machine_type != MachineType.WORKER;
	}

	@Override
	public boolean canReceive() {
		return machine_type != MachineType.GENERATOR;
	}

	@Override
	@Nonnull
	public CompoundTag serializeNBT() {
		CompoundTag tag = new CompoundTag();
		tag.putString("energy_stored", energy_stored.toString());
		tag.putString("energy_stored_max", energy_stored_max.toString());
		tag.putString("max_input", max_input.toString());
		tag.putString("max_output", max_output.toString());
		return tag;
	}

	@Override
	public void deserializeNBT(CompoundTag nbt) {
		if (nbt.contains("energy_stored"))
			energy_stored = new BigDecimal(nbt.getString("energy_stored"));

		if (nbt.contains("energy_stored_max"))
			energy_stored_max = new BigDecimal(nbt.getString("energy_stored_max"));

		if (nbt.contains("max_input"))
			max_input = new BigDecimal(nbt.getString("max_input"));

		if (nbt.contains("max_output"))
			max_output = new BigDecimal(nbt.getString("max_output"));
	}
}
