package ru.dbotthepony.mc.otm.screen.panels;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.gui.components.AbstractWidget;
import net.minecraft.client.gui.screens.Screen;
import net.minecraft.network.chat.Component;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.screen.MatteryScreen;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import static org.lwjgl.opengl.GL11.GL_ALWAYS;

public class MinecraftWidgetPanel<T extends AbstractWidget> extends EditablePanel {
	public interface WidgetFactory<T extends AbstractWidget> {
		T create(MinecraftWidgetPanel<T> panel);
	}

	protected T widget;
	private final WidgetFactory<T> factory;

	@Nullable
	public T getWidget() {
		return widget;
	}

	public MinecraftWidgetPanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, float x, float y, float width, float height, WidgetFactory<T> factory) {
		super(screen, parent, x, y, width, height);
		this.factory = factory;
	}

	protected void copyValues(@Nonnull T new_widget, @Nonnull T old_widget) {

	}

	private void recreate() {
		if (widget == null) {
			return;
		}

		var new_widget = factory.create(this);
		new_widget.visible = getVisible();
		new_widget.x = (int) parent_x;
		new_widget.y = (int) parent_y;

		if (widget != null)
			copyValues(new_widget, widget);

		widget = new_widget;

		factorize(widget, true);
	}

	@Override
	protected void widthUpdated(float new_value, float old_value) {
		recreate();
	}

	@Override
	protected void heightUpdated(float new_value, float old_value) {
		recreate();
	}

	@Override
	protected void visibilityChanges(boolean new_visible, boolean old_visible) {
		if (widget != null)
			widget.visible = new_visible;
	}

	/**
	 * Override in final classes to define values
	 *
	 * @param widget
	 */
	protected void factorize(T widget, boolean recreation) {

	}

	@Nonnull
	public T getOrCreateWidget() {
		if (widget != null)
			return widget;

		widget = factory.create(this);
		widget.visible = getVisible();
		factorize(widget, false);

		return widget;
	}

	@Override
	protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
		RenderSystem.depthFunc(GL_ALWAYS);
		getOrCreateWidget().render(stack, (int) mouse_x, (int) mouse_y, flag);
		RenderSystem.depthFunc(GL_ALWAYS);
	}

	@Override
	protected boolean charTypedInternal(char p_94732_, int p_94733_) {
		return getOrCreateWidget().charTyped(p_94732_, p_94733_);
	}

	@Override
	protected boolean keyPressedInternal(int p_94745_, int p_94746_, int p_94747_) {
		return getOrCreateWidget().keyPressed(p_94745_, p_94746_, p_94747_);
	}

	@Override
	protected boolean keyReleasedInternal(int p_94750_, int p_94751_, int p_94752_) {
		return getOrCreateWidget().keyReleased(p_94750_, p_94751_, p_94752_);
	}

	@Override
	protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
		var pos = screenToLocal(mouse_x, mouse_y);
		return getOrCreateWidget().mouseClicked(pos.x(), pos.y(), flag);
	}

	@Override
	protected boolean mouseReleasedInner(double mouse_x, double mouse_y, int flag) {
		var pos = screenToLocal(mouse_x, mouse_y);
		return getOrCreateWidget().mouseReleased(pos.x(), pos.y(), flag);
	}
}
